# -*- coding: utf-8 -*-

from dataclasses import dataclass

@dataclass
class Notification:
    id: str | None = None
    span: int | None = None
    limit: int | None = 300
    subaccounts: bool = False



# -*- coding: utf-8 -*-

"""Request diagnostic information from a probe or AGENT."""

from . import _utils
from ._utils import API_URL

from dataclasses import asdict


ROUTE = "diagnostics"


def get(token: str, checkid: str, args, customerid: str | None = None) -> dict:
    """Get diagnostic information from a probe or AGENT.

    Args:
        token (str): NodePing API token
        checkid (str): check id associated with the diagnostic request
        args (dataclass): nptypes.checktypes.{Mtr,Ping,Traceroute,Dig,Pageload,Screenshot}
        customerid (str): subaccount ID

    Return:
        dict: Diagnostic information
    """

    querystring = _utils.generate_querystring(asdict(args))
    url = "{}/{}/{}?{}".format(API_URL, ROUTE, checkid, querystring)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)



# -*- coding: utf-8 -*-

""" Helper functions to reduce code reuse and misc other uses
"""

from typing import Any
from nodepingpy.nptypes.contacttypes import *

from time import time
from urllib.parse import urlencode
from urllib.error import HTTPError as httperror
from urllib.request import Request, urlopen

import json


API_URL = "https://api.nodeping.com/api/1"


def add_custid(
    data: dict[str, str | int | bool], customerid: str | None = None
) -> dict:
    """If the customerid isn't None, add it to data

    Args:
        data (dict): data dictionary
        customerid (str): NodePing customerid/subaccount id

    Returns:
        dict: data with customerid included if it isn't None
    """
    if customerid:
        data["customerid"] = customerid

    return data


def generate_querystring(args: dict) -> str:
    """Generates a querystring from a dict.

    Removes all `None` values and escaples invalid strings.

    Args:
        args (dict): key/value for querystring keys and values

    Returns:
        str: escaped querystring
    """

    return urlencode({k: v for k, v in args.items() if v != None})


def create_timestamp(duration):
    return int(time() * 1000) + (duration * 1000)


def get(url: str, data_dict: dict[str, str | int | bool | None]) -> dict:
    """Queries the URL with a GET request with JSON body.

    Does an HTTP GET request and returns an expected JSON payload
    as a Python dictionary.

    Args:
        url (str): URL for the GET request
        data_dict (dict): Dictionary to be submitted as the body

    Returns:
        dict: Data that was returned from NodePing from GET request
    """

    json_data = json.dumps(strip_none_values(data_dict)).encode("utf-8")

    req = Request(url)
    req.get_method = lambda: "GET"
    req.add_header("Content-Type", "application/json; charset=utf-8")
    req.add_header("Content-Length", str(len(json_data)))

    try:
        data = urlopen(req, json_data)
    except httperror as err:
        data = err

    json_bytes = data.read()

    return json.loads(json_bytes.decode("utf-8"))


def post(url: str, data_dict: dict[str, str | int | bool | None]) -> dict:
    """Queries the NodePing API via POST and creates a check

    Accepts a URL and data and POSTs the results to NodePing
    which then creates the check on the account with the user
    specified parameters

    Args:
        url (str): The URL to submit POST to
        data_dict (dict): Dictionary of data that is sent to NodePing

    Returns:
        dict: Response from API
    """

    json_data = json.dumps(strip_none_values(data_dict)).encode("utf-8")

    req = Request(url)
    req.add_header("Content-Type", "application/json; charset=utf-8")
    req.add_header("Content-Length", str(len(json_data)))

    try:
        data = urlopen(req, json_data)
    except httperror as err:
        data = err

    json_bytes = data.read()

    return json.loads(json_bytes.decode("utf-8"))


def put(url: str, data_dict: dict[str, str | int | bool | None]) -> dict:
    """Queries the NodePing API with a PUT request.

    Accepts a URL and data and PUTs the results to NodePing. The
    URL must have a checkid in the URL that will be updated. This
    updates the specified fields in the check.

    Args:
        url (str): The URL to submit POST to
        data_dict (dict): Dictionary of data that is sent to NodePing

    Returns:
        dict: Response from API
    """

    json_data = json.dumps(strip_none_values(data_dict)).encode("utf-8")

    req = Request(url)
    req.get_method = lambda: "PUT"
    req.add_header("Content-Type", "application/json; charset=utf-8")
    req.add_header("Content-Length", str(len(json_data)))

    try:
        data = urlopen(req, json_data)
    except httperror as err:
        data = err

    json_bytes = data.read()

    return json.loads(json_bytes.decode("utf-8"))


def delete(url: str, data_dict: dict[str, str | int | bool]) -> dict[str, Any]:
    """Queries the NodePing API via DELETE and returns its result

    Accepts a URL to the NodePing API to do a delete. A dictionary
    will be returned with "ok" == true meaning it was deleted, if
    false then the check wasn't deleted or an invalid ID was given

    Args:
        url (str): The URL to submit POST to
        data_dict (dict): Dictionary of data that is sent to NodePing

    Returns:
        dict: Response from API
    """

    json_data = json.dumps(strip_none_values(data_dict)).encode("utf-8")

    req = Request(url)
    req.get_method = lambda: "DELETE"
    req.add_header("Content-Type", "application/json; charset=utf-8")
    req.add_header("Content-Length", str(len(json_data)))

    try:
        data = urlopen(req, json_data)
    except httperror as err:
        data = err

    json_bytes = data.read()

    return json.loads(json_bytes.decode("utf-8"))


def strip_none_values(data: dict) -> dict:
    """Remove any keys with a value of None."""
    return {k: v for k, v in data.items() if bool(v) or isinstance(v, bool)}



from . import (
    accounts,
    checks,
    contactgroups,
    contacts,
    diagnostics,
    information,
    maintenance,
    notificationprofiles,
    notifications,
    results,
    schedules,
)
from .nptypes import (
    checktypes,
    contacttypes,
    diagtypes,
    maintenancetypes,
    notificationtypes,
    resulttypes,
)



# -*- coding: utf-8 -*-

"""NodePing check types."""

from dataclasses import dataclass, field
from typing import TypedDict


DEFAULTS = {
    "interval": 15,
    "enabled": False,
    "public": False,
    "runlocations": None,
    "homeloc": False,
    "ipv6": False,
    "threshold": 5,
    "sens": 2,
    "mute": False,
    "autodiag": False,
    "dep": "",
}


@dataclass
class AgentCheck:
    """NodePing AGENT check.

    AGENT checks allow you to install a NodePing probe,
    installed and maintained by you and available only to your account,
    inside your private network that you can assign other NodePing checks
    to run on.

    https://nodeping.com/agent_check.html

    Args:
        oldresultfail (bool): Fail the check if results are too old
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    oldresultfail: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "AGENT"

    def __post_init__(self):
        self.type = "AGENT"


@dataclass
class AudioCheck:
    """NodePing AUDIO check.

    Monitor audio streaming services.

    https://nodeping.com/audio_check.html

    Args:
        target (str): URL to target host
        label (str): Name of the check that will be created
        verifyvolume (bool): enable/disable volume detection
        volumemin (int): The acceptable range for volume detection
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    label: str = ""
    verifyvolume: bool = False
    volumemin: int = -45
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "AUDIO"

    def __post_init__(self):
        self.type = "AUDIO"


@dataclass
class ClusterCheck:
    """NodePing Cluster check.

    Group interdependent NodePing checks in order to get notifications and
    track availability of the group of checks as a whole, in addition to
    each component check individually.

    https://nodeping.com/cluster_check.html

    `data` example:
    checks = {
        "data": {
            "201205050153W2Q4C-0J2HSIRF": "1",
            "201205050153W2Q4C-4RZT8MLN": "1",
            "201205050153W2Q4C-IOPPFQOT": "1"
        }
    }

    Args:
        data (dict): List of checks associated with the cluster
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    data: dict
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "CLUSTER"

    def __post_init__(self):
        self.type = "CLUSTER"


@dataclass
class DnsCheck:
    """NodePing DNS check.

    https://nodeping.com/dns_check.html

    Args:
        target (str): URL of host to monitor
        port (int): Port for DNS server to query
        transport (str): UDP/TCP for DNS query
        dnstype (str): Type of DNS record to query
        dnsrd (int): Recursion desired. 1 for True, 0 for False
        dnssection (str): section of the DNS reply to look in for the `contentstring`
        contentstring (str): What you expect the response to be when resolved
        dnstoresolve (str): FQDN/IP you want to resolve
        verify (bool): If True will authenticate using DNSSEC
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str = ""
    port: int = 53
    transport: str = "udp"
    dnstype: str = "A"
    dnsrd: int = 1
    dnssection: str = "answer"
    contentstring: str = ""
    dnstoresolve: str = ""
    verify: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "DNS"

    def __post_init__(self):
        self.type = "DNS"


@dataclass
class DohDotCheck:
    """NodePing DOHDOT check.

    https://nodeping.com/dohdot_check.html

    Args:
        target (str): URL of host to monitor
        dnstoresolve (str): FQDN to query on target
        dohdot (str): DoH or DoT
        method (str): Select GET or POST
        statuscode (str): Expected Response HTTP status code (DoH only)
        sendheaders (dict): Request Headers (DoH only)
        clientcert (str): string to specify the ID of a client cert/key to be used
        verify (bool): Whether or not to verify the SSL cert
        dnstype (str): Type of DNS record to query
        contentstring (str): What you expect the response to be when resolved
        edns (dict): optional object used to send EDNS(0) OPT pseuedo-records in a DNS query
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    dnstoresolve: str
    dohdot: str = "doh"
    method: str = "GET"
    statuscode: str = "200"
    sendheaders: dict = field(default_factory=dict)
    clientcert: str = ""
    verify: bool = False
    dnstype: str = "A"
    contentstring: str = ""
    edns: dict = field(default_factory=dict)
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "DOHDOT"

    def __post_init__(self):
        self.type = "DOHDOT"


@dataclass
class FtpCheck:
    """NodePing FTP check.

    https://nodeping.com/ftp_check.html

    Args:
        target (str): URL of host to monitor
        port (int): FTP server port
        username (str): Username to authenticate FTP connection
        password (str): Password for user authentication
        invert (bool): Whether you expect the content to exist or not (True == exists)
        contentstring (str): What you expect the response to be when resolved
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 21
    username: str = ""
    password: str = ""
    invert: bool = False
    contentstring: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "FTP"

    def __post_init__(self):
        self.type = "FTP"


@dataclass
class HttpCheck:
    """NodePing HTTP check.

    https://nodeping.com/http_check.html

    Args:
        target (str): URL to target host
        follow (bool): Whether to follow redirects or not
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    follow: bool = False
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "HTTP"

    def __post_init__(self):
        self.type = "HTTP"


@dataclass
class HttpAdvCheck:
    """NodePing HTTPADV check.

    https://nodeping.com/http_advanced_check.html

    Args:
        target (str): URL to target host
        invert (bool): Used for "Does not contain" functionality
        contentstring (str): What you expect the response to be when resolved
        data (dict): key/value pair for POST fields
        clientcert (str): Specify the ID of a client certificate/key to be used
        method (str): HTTP method
        postdata (str): a single string to post instead of a data dictionary
        receiveheaders (dict): Headers that should be received
        sendheaders (dict): Headers to send in request
        statuscode (int): HTTP status code expected in return
        follow (bool): Whether to follow redirects or not
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    invert: bool = False
    contentstring: str = ""
    data: dict = field(default_factory=dict)
    clientcert: str = ""
    method: str = ""
    postdata: str = ""
    receiveheaders: str = ""
    sendheaders: str = ""
    statuscode: int = 200
    follow: bool = False
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "HTTPADV"

    def __post_init__(self):
        self.type = "HTTPADV"


@dataclass
class HttpContentCheck:
    """NodePing HTTPCONTENT check.

    https://nodeping.com/http_content_check.html

    Args:
        target (str): URL to target host
        invert (bool): Used for "Does not contain" functionality
        contentstring (str): What you expect the response to be when resolved
        follow (bool): Whether to follow redirects or not
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    invert: bool = False
    contentstring: str = ""
    follow: bool = False
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "HTTPCONTENT"

    def __post_init__(self):
        self.type = "HTTPCONTENT"


@dataclass
class HttpParseCheck:
    """NodePing HTTPPARSE check.

    https://nodeping.com/http_content_check.html

    `fields` expects a dictionary where each object should have a name, min, and max.

    Example dictionary:
    fields = {
        "processmem": {
            "name": "processmem",
            "min": 1000,
            "max": 5000
        },
        "cpuload": {
            "name": "cpuload",
            "min": 1,
            "max": 5
        }
    }

    Args:
        target (str): URL to target host
        fields (dict): Keyed list of fields, with an arbitrary string as the key.
        sendheaders (dict): Headers to send in request
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    fields: dict = field(default_factory=dict)
    sendheaders: dict = field(default_factory=dict)
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "HTTPPARSE"

    def __post_init__(self):
        self.type = "HTTPPARSE"


@dataclass
class Imap4Check:
    """NodePing IMAP4 check.

    https://nodeping.com/imap_check.html

    Args:
        target (str): URL to target host
        port (int): port used to test IMAP4 communications
        verify (bool): The check should fail if the SSL/TLS certificate is invalid
        username (str): Email address for testing logins
        password (str): Password to authenticate email address
        secure (bool): Whether SSL/TLS should be used
        warningdays (int): Warning days befor certificate expiration
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 143
    verify: bool = True
    username: str = ""
    password: str = ""
    secure: bool = False
    warningdays: int = 0
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "IMAP4"

    def __post_init__(self):
        self.type = "IMAP4"


@dataclass
class MongodbCheck:
    """NodePing MONGODB check.

    https://nodeping.com/mongodb_check.html

    `fields` example:

    fields = {
        "processmem": {
            "name": "processmem",
            "min": 1000,
            "max": 5000
        },
        "cpuload": {
            "name": "cpuload",
            "min": 1,
            "max": 5
        }
    }

    Args:
        target (str): URL to target host
        query (str): JSON query to send to the database server (limit 25 results)
        fields (dict): Queries results to match.
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    query: str = ""
    fields: dict = field(default_factory=dict)
    database: str = ""
    namespace: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "MONGODB"

    def __post_init__(self):
        self.type = "MONGODB"


@dataclass
class MtrCheck:
    """NodePing MTR check.

    https://nodeping.com/mtr_check.html

    Args:
        target (str): URL to target host
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "MTR"

    def __post_init__(self):
        self.type = "MTR"


@dataclass
class MySqlCheck:
    """NodePing MYSQL check.

    https://nodeping.com/mysql_check.html

    `fields` example:

    fields = {
        "checknum": {
            "name": "checknum",
            "min": 0,
            "max": 5
        },
        "checkstring": {
            "name": "checkstring",
            "match": "exactmatch"
        }
    }

    Args:
        target (str): URL to target host
        port (int): port used to test communications
        username (str): username for testing logins
        password (str): Password to authenticate to database
        secure (bool): Whether SSL/TLS should be used
        database (str): name of database to query
        query (str): SQL query to send to the database server
        fields (dict): Queries results to match.
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 3306
    username: str = ""
    password: str = ""
    secure: bool = False
    database: str = ""
    query: str = ""
    fields: dict = field(default_factory=dict)
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "MYSQL"

    def __post_init__(self):
        self.type = "MYSQL"


@dataclass
class NtpCheck:
    """NodePing NTP check.

    https://nodeping.com/ntp_check.html

    Args:
        target (str): URL to target host
        port (int): port used to test communications
        invert (bool): Pass if it responds (True) or not (False)
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 123
    invert: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "NTP"

    def __post_init__(self):
        self.type = "NTP"


@dataclass
class PingCheck:
    """NodePing PING check.

    https://nodeping.com/ping_check.html

    Args:
        target (str): URL to target host
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "PING"

    def __post_init__(self):
        self.type = "PING"


@dataclass
class Pop3Check:
    """NodePing POP3 check.

    https://nodeping.com/pop_check.html

    Args:
        target (str): URL to target host
        port (int): port used to test POP3 communications
        verify (bool): The check should fail if the SSL/TLS certificate is invalid
        username (str): Email address for testing logins
        password (str): Password to authenticate email address
        secure (bool): Whether SSL/TLS should be used
        warningdays (int): Warning days befor certificate expiration
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 110
    verify: bool = True
    username: str = ""
    password: str = ""
    secure: bool = False
    warningdays: int = 0
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "POP3"

    def __post_init__(self):
        self.type = "POP3"


@dataclass
class PortCheck:
    """NodePing PORT check.

    https://nodeping.com/port_check.html

    Args:
        target (str): URL to target host
        port (int): Port to check
        invert (bool): True if accepts connection, False if does not accept connection to pass
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int
    invert: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "PORT"

    def __post_init__(self):
        self.type = "PORT"


@dataclass
class PostgreSqlCheck:
    """NodePing PGSQL check.

    https://nodeping.com/pgsql_check.html

    `fields` example:

    fields = {
        "checknum": {
            "name": "checknum",
            "min": 0,
            "max": 5
        },
        "checkstring": {
            "name": "checkstring",
            "match": "exactmatch"
        }
    }

    Args:
        target (str): URL to target host
        query (str): SQL query to send to the database server
        fields (dict): Queries results to match.
        ipv6 (bool): Whether to resolve IPv4 or IPv6
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    query: str = ""
    fields: dict = field(default_factory=dict)
    ipv6: bool = False
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "PGSQL"

    def __post_init__(self):
        self.type = "PGSQL"


@dataclass
class PushCheck:
    """NodePing PUSH check.

    https://nodeping.com/push_check.html

    `fields` example:

    fields = {
        "checknum": {
            "name": "checknum",
            "min": 0,
            "max": 5
        },
        "check2": {
            "name": "check2.item",
            "min": 0,
            "max": 0
        }
    }

    Args:
        checktoken (str): "reset" to regenerate the checktoken, or set to an empty string
        fields (dict): Contents of each metric collected with min/max values
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    checktoken: str = "reset"
    fields: dict = field(default_factory=dict)
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "PUSH"

    def __post_init__(self):
        self.type = "PUSH"


@dataclass
class RblCheck:
    """NodePing RBL check.

    https://nodeping.com/rbl_check.html

    Args:
        target (str): URL to target host
        ignore (str): comma separated list of RBLs to ignore
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    ignore: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "RBL"

    def __post_init__(self):
        self.type = "RBL"


@dataclass
class RedisCheck:
    """NodePing REDIS check.

    https://nodeping.com/redis_check.html

    Args:
        target (str): URL to target host
        redistype (str): standalone, sentinel, cluster
        hosts (list): For sentinel or cluster. The hostname, port, and password to connect
        sentinelname (str): The master/primary name for the sentinel to query
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    redistype: str
    sentinelname: str = ""
    hosts: list = field(default_factory=list)
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "REDIS"

    def __post_init__(self):
        self.type = "REDIS"


@dataclass
class RdpCheck:
    """NodePing RDP check.

    Args:
        target (str): URL to target host
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "RDP"

    def __post_init__(self):
        self.type = "RDP"


@dataclass
class SipCheck:
    """NodePing SIP check.

    https://nodeping.com/sip_check.html

    Args:
        target (str): URL to target host
        transport (str): udp, tcp, tls, ws, or wss transport protocol options
        ignore (str): comma-separated list of status codes to ignore
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    transport: str = "udp"
    ignore: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SIP"

    def __post_init__(self):
        self.type = "SIP"


@dataclass
class SmtpCheck:
    """NodePing SMTP check.

    https://nodeping.com/smtp_check.html

    Args:
        target (str): URL to target host
        port (int): port used to test POP3 communications
        invert (bool): whether or not SMTP accepts mail from the email address
        verify (bool): The check should fail if the SSL/TLS certificate is invalid
        email (str): Email address that will be used to test smtp connectivity
        username (str): Email address for testing logins
        password (str): Password to authenticate email address
        secure (bool): Whether SSL/TLS should be used
        warningdays (int): Warning days befor certificate expiration
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 25
    invert: bool = False
    verify: bool = True
    email: str = ""
    username: str = ""
    password: str = ""
    secure: bool = False
    warningdays: int = 0
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SMTP"

    def __post_init__(self):
        self.type = "SMTP"


@dataclass
class SnmpCheck:
    """NodePing SNMP check.

    https://nodeping.com/snmp_check.html

    Args:
        target (str): URL to target host
        port (int): port used to test POP3 communications
        invert (bool): whether or not SMTP accepts mail from the email address
        verify (bool): The check should fail if the SSL/TLS certificate is invalid
        email (str): Email address that will be used to test smtp connectivity
        username (str): Email address for testing logins
        password (str): Password to authenticate email address
        secure (bool): Whether SSL/TLS should be used
        warningdays (int): Warning days befor certificate expiration
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    port: int = 161
    fields: dict = field(default_factory=dict)
    snmpv: int = 1
    snmpcom: str = "public"
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SNMP"

    def __post_init__(self):
        self.type = "SNMP"


@dataclass
class Spec10DnsCheck:
    """NodePing SPEC10DNS check.

    https://nodeping.com/spec10dns_check.html

    Example `data` dictionary:
    checks = {
        "data": {
            "201205050153W2Q4C-0J2HSIRF": "1",
            "201205050153W2Q4C-4RZT8MLN": "1",
            "201205050153W2Q4C-IOPPFQOT": "1"
        }
    }

    Args:
        data (dict): dictonary of child DNS checks
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    data: dict
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SPEC10DNS"

    def __post_init__(self):
        self.type = "SPEC10DNS"


@dataclass
class Spec10RddsCheck:
    """NodePing SPEC10RDDS check.

    https://nodeping.com/spec10rdds_check.html

    Example `data` dictionary:
    checks = {
        "data": {
            "201205050153W2Q4C-0J2HSIRF": "1",
            "201205050153W2Q4C-4RZT8MLN": "1",
            "201205050153W2Q4C-IOPPFQOT": "1"
        }
    }

    Args:
        data (dict): dictonary of child WHOIS checks
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    data: dict
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SPEC10RDDS"

    def __post_init__(self):
        self.type = "SPEC10RDDS"


@dataclass
class SshCheck:
    """NodePing SSH check.

    https://nodeping.com/ssh_check.html

    Args:
        target (str): URL or IP to target host
        invert (bool): whether or not the SSH connection is established or not
        contentstring (str): Check that the SSH connection returns this string content
        port (int): sshd port to test
        username (str): SSH username for testing logins
        password (str): Password to authenticate user
        sshkey (str): specify the ID of a SSH private key to be used in the SSH check
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    invert: bool = False
    contentstring: str = ""
    port: int = 22
    username: str = ""
    password: str = ""
    sshkey: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SSH"

    def __post_init__(self):
        self.type = "SSH"


@dataclass
class SslCheck:
    """NodePing SSL check.

    https://nodeping.com/ssl_check.html

    Args:
        target (str): URL or IP to target host
        warningdays (int): number of days before to warn of an expiring SSL cert
        servername (str): FQDN sent to SNI services in the SSL check
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    warningdays: int = 0
    servername: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "SSL"

    def __post_init__(self):
        self.type = "SSL"


@dataclass
class WebsocketCheck:
    """NodePing WEBSOCKET check.

    https://nodeping.com/websocket_check.html

    Args:
        target (str): URL or IP to target host
        invert (bool): Whether or not the response contains the contentstring
        contentstring (str): string content expected in the response
        data (str): Content to send. Can be JSON, XML, TXT, etc. Whatever the server understands
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    invert: bool = False
    contentstring: str = ""
    data: str = ""
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "WEBSOCKET"

    def __post_init__(self):
        self.type = "WEBSOCKET"


@dataclass
class WhoisCheck:
    """NodePing WHOIS check.

    https://nodeping.com/whois_check.html

    Args:
        target (str): The FQDN to check
        whoisserver (str): Server to query WHOIS information from
        invert (bool): Whether or not the response contains the contentstring
        contentstring (str): string content expected in the response
        warningdays (int): number of days to warn of domain registration expiration
        label (str): Name of the check that will be created
        autodiag (bool): Enable/disable auto diagnostics for this check
        interval (int): Interval in minutes to monitor target
        enabled (bool): If created check will be enabled or disabled
        public (bool): If the results for the created check will be public or not
        runlocations (str|list): Which region to be originated from
        homeloc (str|bool): Which probe in the region to originate the check from
        threshold (int): Time in seconds for an acceptable response
        sens (int): Rechecks to help avoid unecessary notifications
        dep (str): ID of the check used for the notification dependency
        notifications (list): list of objects containing contact ID, delay, and scheduling for notifications
        mute (bool|int): bool or millisecond timestamp (UTC) in the future. True to mute indefinitely
    """

    target: str
    whoisserver: str = ""
    ipv6: bool = False
    invert: bool = False
    contentstring: str = ""
    warningdays: int = 0
    label: str = ""
    autodiag: bool = DEFAULTS["autodiag"]
    interval: int = DEFAULTS["interval"]
    enabled: bool = DEFAULTS["enabled"]
    public: bool = DEFAULTS["enabled"]
    runlocations: str | list = DEFAULTS["runlocations"]
    homeloc: str | bool = DEFAULTS["homeloc"]
    threshold: int = DEFAULTS["threshold"]
    sens: int = DEFAULTS["sens"]
    dep: str = DEFAULTS["dep"]
    notifications: list = field(default_factory=list)
    mute: bool = DEFAULTS["mute"]
    type: str = "WHOIS"

    def __post_init__(self):
        self.type = "WHOIS"


class GetCheck(TypedDict):
    _id: str
    autodiag: bool
    created: int
    customer_id: str
    dep: bool
    enable: str
    firstdown: int
    homeloc: bool | str
    interval: int | float
    label: str
    modified: int
    mute: bool | int
    notifications: list
    parameters: dict
    public: bool
    queue: str
    runlocations: bool
    state: int
    status: str
    suspacct: bool
    type: str
    uuid: str

class UptimeInfo(TypedDict):
    down: str | int
    enabled: int
    uptime: float

class GetCheckUptime(GetCheck):
    uptime: dict[str, UptimeInfo]

class ModifiedCheck(TypedDict):
    _id: str
    autodiag: bool
    change: int
    created: int
    customer_id: str
    dep: bool
    enable: str
    homeloc: bool | str
    interval: int | float
    label: str
    modified: int
    mute: bool | int
    notifications: list
    parameters: dict
    public: bool
    runlocations: bool
    status: str
    type: str
    uuid: str



# -*- coding: utf-8 -*-

"""NodePing maintenance dataclass."""

from dataclasses import dataclass


@dataclass
class AdHocCreate:
    """Args for creating an ad-hoc maintenance.

    https://nodeping.com/docs-api-maintenance.html

    Args:
        duration (int): duration in minutes to keep the checks disabled
        checklist (list): List of check IDs to disable
        enabled (bool): whether or not to enable or disable this maintenance
        name (str): name for the ad-hoc maintenance
    """
    duration: int
    checklist: list
    enabled: bool
    name: str
    id: str = "ad-hoc"

    def __post_init__(self):
        self.id = "ad-hoc"


@dataclass
class AdHocUpdate:
    """Args for updating an ad-hoc maintenance.

    https://nodeping.com/docs-api-maintenance.html

    Args:
        duration (int): duration in minutes to keep the checks disabled
        checklist (list): List of check IDs to disable
        enabled (bool): optional - whether or not to enable or disable this maintenance
        name (str): optional - new name for the ad-hoc maintenance
    """
    duration: int
    checklist: list
    enabled: bool | None = None
    name: str | None = None
    id: str = "ad-hoc"

    def __post_init__(self):
        self.id = "ad-hoc"


@dataclass
class ScheduledCreate:
    """Args for creating a scheduled maintenance.

    https://nodeping.com/docs-api-maintenance.html

    Args:
        duration (int): duration in minutes to keep the checks disabled
        checklist (list): List of check IDs to disable
        enabled (bool): whether or not to enable or disable this maintenance
        name (str): name for the scheduled maintenance
        cron (str): cron line for scheduled maintenance start
    """
    duration: int
    checklist: list
    enabled: bool
    name: str
    cron: str


@dataclass
class ScheduledUpdate:
    """Args for updating a scheduled maintenance.

    https://nodeping.com/docs-api-maintenance.html

    Args:
        duration (int): duration in minutes to keep the checks disabled
        checklist (list): List of check IDs to disable
        enabled (bool): optional - whether or not to enable or disable this maintenance
        name (str): optional - new name for the scheduled maintenance
        cron (str): optional - new cron line for scheduled maintenance start
    """
    duration: int
    checklist: list
    enabled: bool | None = None
    name: str | None = None
    cron: str | None = None



# -*- coding: utf-8 -*-

""" Manage contacts on your NodePing account."""


from .nptypes import contacttypes
from . import _utils
from ._utils import API_URL


ROUTE = "contacts"


def get_all(
    token: str, customerid: str | None = None
) -> dict[str, contacttypes.ManyContacts]:
    """Get all contacts on the account or subaccount.

    https://nodeping.com/docs-api-contacts.html#get

    Args:
        token (str): NodePing API token
        customerid (str | None): subaccount ID

    Returns:
        dict: All contacts on NodePing account or subaccount.
    """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}".format(API_URL, ROUTE), data)


def get_one(
    token: str, contactid: str, customerid: str | None = None
) -> dict[str, contacttypes.Contact]:
    """Get one contact on the account or subaccount.

    https://nodeping.com/docs-api-contacts.html#get

    Args:
        token (str): NodePing API token
        contactid (str): The `_id` for the contact
        customerid (str | None): subaccount ID

    Returns:
        dict: All contacts on NodePing account or subaccount.
    """
    url = "{}/{}?{}".format(
        API_URL, ROUTE, _utils.generate_querystring({"id": contactid})
    )
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def get_by_type(
    token: str, contacttype: str, customerid: str | None = None
) -> dict[str, contacttypes.Contact]:
    """Get one contact on the account or subaccount.

    https://nodeping.com/docs-api-contacts.html#get

    Args:
        token (str): NodePing API token
        contacttype (str): sms, email, webhook, etc.
        customerid (str | None): subaccount ID

    Returns:
        dict: All contacts on NodePing account or subaccount of type `contacttype`
    """
    contacts_dict = {}
    contacts = get_all(token, customerid)

    for key, value in contacts.items():
        for _, contents in value["addresses"].items():
            # sometimes type may not exist
            try:
                if contents["type"] == contacttype:
                    contacts_dict.update({key: value})
            except KeyError:
                continue

    return contacts_dict


def create(
    token: str,
    customerid: str,
    custrole: str,
    name: str = "",
    newaddresses: list | None = None,
) -> dict:
    """Create a new contact on your account or subaccount.

    https://nodeping.com/docs-api-contacts.html#post-put

    newaddresses example:
    [{'address': 'me@email.com', 'type': 'email'}, {'address': '5551238888', 'type': 'sms'}]

    newaddresses webhook example

    [{'action': 'post',
        'address': 'https://webhook.example.com',
        'data': {'event': '{event}',
            'id': '{_id}',
            'label': '{label}',
            'runtime': '{runtime}',
            'target': '{target}'},
        'headers': {'Content-Type': 'application/json'},
        'type': 'webhook'}
    ]

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID
        contacttype (str): sms, email, webhook, etc.
        name (str): The name of your contact
        custrole (str): permissions for this contact. Default: view
        newaddresses (str): list of dictionaries containing address info

    Returns:
        dict: API result about created contact
    """
    data = {
        "name": name,
        "newaddresses": newaddresses,
        "custrole": custrole,
        "token": token,
        "customerid": customerid,
    }

    return _utils.post("{}/{}/{}".format(API_URL, ROUTE, customerid), data)


def update(token: str, cid: str, args: dict, customerid: str | None = None) -> dict:
    """Update an existing contact.

    NOTE: If you are using the addresses argument to update an existing
    address, you must supply the entire list of contacts. For example, if
    a contact has 2 addresses and you are only updating one, you must supply
    both addresses, one being with the revisions you are planning to make.

    Args:
        token (str): NodePing API token
        cid (str): The contact ID of the contact that is being changed
        customerid (str | None): subaccount ID

    Returns:
        dict: The contents of the updated contact
    """
    data = _utils.add_custid(args, customerid)
    data["token"] = token
    data["id"] = cid

    return _utils.put("{}/{}/{}".format(API_URL, ROUTE, cid), data)


def mute_contact(
    token: str,
    contact_dict: dict,
    duration: int | bool,
    customerid: str | None = None,
) -> dict:
    """Mute a contact.

    Mute or unmute a contact indefinitely, or mute it up until a set timestamp.
    Note that to update the contact, you must return the whole contact,
    otherwise you will lose contact methods in the update.

    Args:
        token (str): NodePing API token
        contact_dict (dict): The entire dict for the contact.
        duration (int|bool): true to mute infinitely, false to unmute, or a unix timestamp
        customerid (str | None): subaccount ID

    Returns:
        dict: The contents of the contact with the applied mute information
    """
    for _, address in contact_dict["addresses"].items():
        address["mute"] = duration

    addresses = contact_dict["addresses"]
    data = _utils.add_custid(
        {"token": token, "addresses": addresses}, customerid
    )

    return _utils.put("{}/{}/{}".format(API_URL, ROUTE, contact_dict["_id"]), data)


def mute_contact_method(
    token: str,
    contact_dict: dict,
    method_id: str,
    duration: int | bool,
    customerid: str | None = None,
):
    """Mute a contact method of a contact.

    Mute or unmute a single contact method indefinitely, or mute it up until a set timestamp.
    Note that to update the contact, you must return the whole contact,
    otherwise you will lose contact methods in the update.

    Args:
        token (str): NodePing API token
        contact_dict (dict): The entire dict for the contact, such as one fetched with get_one()
        method_id (str): the contact method id (e.g - K5SP9CQP found in the addresses object)
        duration (int|bool): true to mute infinitely, false to unmute, or a unix timestamp
        customerid (str | None): subaccount ID

    Returns:
        dict: The contents of the contact with the applied mute information
    """
    contact_dict["addresses"][method_id]["mute"] = duration 
    addresses = contact_dict["addresses"]
    data = _utils.add_custid({"token": token, "addresses": addresses}, customerid)

    return _utils.put("{}/{}/{}".format(API_URL, ROUTE, contact_dict["_id"]), data)


def delete_contact(token: str, cid: str, customerid: str | None = None) -> dict:
    """Delete a contact on a NodePing account.

     Args:
        token (str): NodePing API token
        cid (str): the contact id
        customerid (str | None): subaccount ID

    Returns:
        dict: Confirmation of a sucessful or failed deletion.
    """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.delete("{}/{}/{}".format(API_URL, ROUTE, cid), data)


def reset_password(token: str, cid: str, customerid: str | None = None) -> dict:
    """Reset the password for the specified contact.

    You can get the contact_id by querying the API with the get_all
    function. The ID would look something like this: "201205050153W2Q4C-OVDN7"

    Args:
        token (str): NodePing API token
        cid (str): the contact id
        customerid (str | None): subaccount ID

    Returns:
        dict: Confirmation of a successful or failed password reset.
    """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}/{}?action=RESETPASSWORD".format(API_URL, ROUTE, cid), data)



# -*- coding: utf-8 -*-

"""NodePing diagnostic dataclass."""

from dataclasses import dataclass


@dataclass
class Mtr:
    """Args for diagnostics.

    https://nodeping.com/docs-api-diagnostics.html

    Args:
        location (str): probe 2-character indicater or AGENT check ID
        target (str | None): URL, FQDN, IP address to get diagnostics about. Taken from check data if not specified
        count (int): number of pings to send
    """

    location: str
    target: str | None = None
    count: int = 10
    tool: str = "mtr"

    def __post_init__(self):
        self.tool = "mtr"


@dataclass
class Ping:
    """Args for diagnostics.

    https://nodeping.com/docs-api-diagnostics.html

    Args:
        location (str): probe 2-character indicater or AGENT check ID
        target (str | None): URL, FQDN, IP address to get diagnostics about. Taken from check data if not specified
        count (int): number of pings to send
    """

    location: str
    target: str | None = None
    count: int = 10
    tool: str = "ping"

    def __post_init__(self):
        self.tool = "ping"


@dataclass
class Traceroute:
    """Args for diagnostics.

    https://nodeping.com/docs-api-diagnostics.html

    Args:
        location (str): probe 2-character indicater or AGENT check ID
        target (str | None): URL, FQDN, IP address to get diagnostics about. Taken from check data if not specified
    """

    location: str
    target: str | None = None
    tool: str = "traceroute"

    def __post_init__(self):
        self.tool = "traceroute"


@dataclass
class Dig:
    """Args for diagnostics.

    https://nodeping.com/docs-api-diagnostics.html

    Args:
        location (str): probe 2-character indicater or AGENT check ID
        target (str | None): URL, FQDN, IP address to get diagnostics about. Taken from check data if not specified
        dnsserver (str): FQDN or IP of the DNS server to query
        dnstype (str): DNS record type for the query
        transport (str): udp or tcp
    """

    location: str
    target: str | None = None
    dnsserver: str | None = None
    dnstype: str = "A"
    transport: str = "udp"
    tool: str = "dig"

    def __post_init__(self):
        self.tool = "dig"


@dataclass
class Pageload:
    """Args for diagnostics.

    https://nodeping.com/docs-api-diagnostics.html

    Args:
        location (str): probe 2-character indicater or AGENT check ID
        target (str | None): URL, FQDN, IP address to get diagnostics about. Taken from check data if not specified
    """

    location: str
    target: str | None = None
    tool: str = "pageload"

    def __post_init__(self):
        self.tool = "pageload"


@dataclass
class Screenshot:
    """Args for diagnostics.

    https://nodeping.com/docs-api-diagnostics.html

    Args:
        location (str): probe 2-character indicater or AGENT check ID
        target (str | None): URL, FQDN, IP address to get diagnostics about. Taken from check data if not specified
    """

    location: str
    target: str | None = None
    tool: str = "screenshot"

    def __post_init__(self):
        self.tool = "screenshot"



# -*- coding: utf-8 -*-

"""Get notifications for checks on your NodePing account.

https://nodeping.com/docs-api-notifications.html
"""


from dataclasses import dataclass, asdict
from . import _utils
from ._utils import API_URL

ROUTE = "notifications"


@dataclass
class Notification:
    """
    Args:
        id (str): Check id of the check for which you want to list notifications.
        span (int): number of hours of notifications to retrieve.
        limit (int): default 300, max 43201 - number of records to retrieve.
        subaccounts (bool): if set, notifications sent to subaccounts will also be included.
    """
    id: str | None = None
    span: int | None = None
    limit: int | None = 300
    subaccounts: bool = False


def get(token: str, args: dict, customerid: str | None = None) -> dict:
    """Get notifications on your NodePing account.

    Get notifications according to a maximum limit (limit caps at 43201),
    or over a certain amount of hours, and whether or not to include
    subaccounts or not when fetching notifications.

    https://nodeping.com/docs-api-notifications.html

    Args:
        token (str): NodePing API token
        args (dict): Notifications class in this module.
        customerid (str): subaccount ID
    """
    data = asdict(args)
    data["token"] = token
    data["customerid"] = customerid

    if customerid:
        url = "{}/{}/{}".format(API_URL, ROUTE, customerid)
    else:
        url = "{}/{}".format(API_URL, ROUTE)

    return _utils.get(url, data)



# -*- coding: utf-8 -*-

""" Manage contact group information."""

from . import _utils
from ._utils import API_URL

ROUTE = "contactgroups"


def get_all(token: str, customerid : str | None = None) -> dict:
    """Get all contact groups on the account or subaccount.

    https://nodeping.com/docs-api-contactgroups.html#get

    Args:
        token (str): NodePing API token
        customerid (str | None): subaccount ID

    Returns:
        dict: All contact groups on the account or subaccount.
    """
    url = "{}/{}".format(API_URL, ROUTE)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def get(token: str, id, customerid : str | None = None) -> dict:
    """Get a contact group on the account or subaccount.

    https://nodeping.com/docs-api-contactgroups.html#get

    Args:
        token (str): NodePing API token
        id (str): The ID for the contact group
        customerid (str | None): subaccount ID

    Returns:
        dict: Information for the contact groups ID.
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, id)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def create(token: str, name: str, members: list[str], customerid: str | None = None) -> dict:
    """Create a new contact group.

    https://nodeping.com/docs-api-contactgroups.html#post-put

    Args:
        token (str): NodePing API token
        name (str): Name for the contact group
        members (list): A list of contact IDs
        customerid (str | None): subaccount ID

    Returns:
        dict: Information about the newly created contact group.
    """
    url = "{}/{}".format(API_URL, ROUTE)
    data = _utils.add_custid({"token": token, "name": name, "members": members}, customerid)

    return _utils.post(url, data)


def update(token: str, id: str, args: dict, customerid: str | None = None) -> dict:
    """Update an existing contact group.

    https://nodeping.com/docs-api-contactgroups.html#post-put

    Args:
        token (str): NodePing API token
        name (str): Name for the contact group
        args (dict): `name` (str) or `members` (list)
        customerid (str | None): subaccount ID

    Returns:
        dict: Information about the updated contact group.
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, id)
    args["token"] = token
    args = _utils.add_custid(args, customerid)

    return _utils.put(url, args)


def delete(token: str, id, customerid : str | None = None) -> dict:
    """Delete an existing contact group.

    https://nodeping.com/docs-api-contactgroups.html#delete

    Args:
        token (str): NodePing API token
        id (str): The ID for the contact group
        customerid (str | None): subaccount ID

    Returns:
        dict: success or failure message for deleting the contact group.
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, id)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.delete(url, data)




# -*- coding: utf-8 -*-

"""
Manage notification profiles for NodePing Account.

https://nodeping.com/docs-api-notificationprofiles.html
"""

from . import _utils
from ._utils import API_URL

ROUTE = "notificationprofiles"


def get_all(token: str, customerid: str | None = None) -> dict:
    """Get all notification profiles on the account.

    https://nodeping.com/docs-api-notificationprofiles.html#get

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: All notification profiles.
    """
    data = _utils.add_custid({"token": token}, customerid)
    url = "{}/{}".format(API_URL, ROUTE)

    return _utils.get(url, data)


def get(token: str, id: str, customerid: str | None = None) -> dict:
    """Get one notification profile on the account.

    https://nodeping.com/docs-api-notificationprofiles.html#get

    Args:
        token (str): NodePing API token
        id (str): The notification profile ID
        customerid (str): subaccount ID

    Returns:
        dict: Get the specified notification profile.
    """
    data = _utils.add_custid({"token": token, "id": id}, customerid)
    url = "{}/{}".format(API_URL, ROUTE)

    return _utils.get(url, data)


def create(
    token: str,
    name: str,
    notifications: list[dict[str, str | int]],
    customerid: str | None = None,
) -> dict:
    """Create a notification profile.

    https://nodeping.com/docs-api-notificationprofiles.html#post-put

    Args:
        token (str): NodePing API token
        name (str): Name of the notification profile.
        notifications (list): List of NodePing contacts and the notification schedules
        customerid (str): subaccount ID

    notifications argument example:
    [
        {"contactkey1":
            {"delay":0,
                "schedule":"schedule1"
            }
        },
        {"contactkey2":
            {"delay":5,
                "schedule":"schedule2"
            }
        }
    ]

    Returns:
        dict: New notification profile information.
    """
    data = _utils.add_custid(
        {"token": token, "name": name, "notifications": notifications}, customerid
    )
    url = "{}/{}".format(API_URL, ROUTE)

    return _utils.post(url, data)


def update(
    token: str,
    id: str,
    name: str,
    notifications: list[dict[str, str | int]],
    customerid: str | None = None,
) -> dict:
    """Update a notification profile.

    https://nodeping.com/docs-api-notificationprofiles.html#post-put

    Args:
        token (str): NodePing API token
        id (str): The notification profile ID
        name (str): Name of the notification profile.
        notifications (list): List of NodePing contacts and the notification schedules
        customerid (str): subaccount ID

    notifications argument example:
    [
        {"contactkey1":
            {"delay":0,
                "schedule":"schedule1"
            }
        },
        {"contactkey2":
            {"delay":5,
                "schedule":"schedule2"
            }
        }
    ]

    Returns:
        dict: Updated notification profile.
    """
    data = _utils.add_custid(
            {"token": token, "name": name, "notifications": notifications, "id": id}, customerid
    )
    url = "{}/{}".format(API_URL, ROUTE)

    return _utils.put(url, data)


def delete(token: str, id: str, customerid: str | None = None) -> dict:
    """Delete one notification profile on the account.

    https://nodeping.com/docs-api-notificationprofiles.html#get

    Args:
        token (str): NodePing API token
        id (str): The notification profile ID
        customerid (str): subaccount ID

    Returns:
        dict: Confirmation whether the deletion was successful or not.
    """
    data = _utils.add_custid({"token": token, "id": id}, customerid)
    url = "{}/{}".format(API_URL, ROUTE)

    return _utils.delete(url, data)





# -*- coding: utf-8 -*-

""" Get checks that were created on your NodePing account.

Allows you go get all checks, get passing, failing, by its ID,
disabled checks, and last results for a check.
"""

from dataclasses import asdict

from .nptypes import checktypes
from . import _utils
from ._utils import API_URL


ROUTE = "checks"


def get_all(
    token: str, customerid: str | None = None
) -> dict[str, checktypes.GetCheck]:
    """Get all checks that exist for the account or subaccount.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: All checks on NodePing account or subaccount.
    """
    url = "{}/{}".format(API_URL, ROUTE)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def get_all_uptime(
    token: str, customerid: str | None = None
) -> dict[str, checktypes.GetCheckUptime]:
    """Get the uptime for all checks on the account or subaccount.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: All checks on NodePing account or subaccount.
    """
    url = "{}/{}".format(API_URL, ROUTE)
    data = _utils.add_custid({"token": token, "uptime": True}, customerid)

    return _utils.get(url, data)


def get_many(
    token: str,
    checkids: list[str],
    customerid: str | None = None,
    current: str | None = None,
) -> dict[str, checktypes.GetCheck]:
    """Get information for all specified checks.

    Args:
        token (str): NodePing API token
        checkids (list): List of NodePing check IDs
        customerid (str): subaccount ID
        current (bool): checks current events

    Additional information about `current` argument:
    https://nodeping.com/docs-api-checks.html

    Returns:
        dict: All specified checks on NodePing account or subaccount.
    """
    url = "{}/{}?{}".format(
        API_URL, ROUTE, _utils.generate_querystring({"id": ",".join(checkids)})
    )
    data = _utils.add_custid({"token": token}, customerid)

    if current:
        data["current"] = current

    return _utils.get(url, data)


def get_passing(
    token: str, customerid: str | None = None
) -> dict[str, checktypes.GetCheck]:
    """Get active passing NodePing checks.

     Args:
        token (str): NodePing API token
        checkids (list): List of NodePing check IDs
        customerid (str): subaccount ID

    Returns:
        dict: All passing checks on NodePing account or subaccount.
    """

    return _parse_pass_fail(get_all(token, customerid), 1)


def get_failing(
    token: str, customerid: str | None = None
) -> dict[str, checktypes.GetCheck]:
    """Get active failing NodePing checks.

     Args:
        token (str): NodePing API token
        checkids (list): List of NodePing check IDs
        customerid (str): subaccount ID

    Returns:
        dict: All failing checks on NodePing account or subaccount.
    """

    return _parse_pass_fail(get_all(token, customerid), 0)


def get_uptime(
    token: str,
    checks: str | list,
    customerid: str | None = None,
    start: str | None = None,
) -> dict[str, checktypes.GetCheckUptime]:
    """Get the uptime for passing and active checks on the account or subaccount.

    Args:
        token (str): NodePing API token
        checks (str|list): "all" for all checks or a list of checks to get
        customerid (str): subaccount ID
        start (str): an optional %Y-%m-%d timestamp of when to start getting uptime results

    Returns:
        dict: All passing checks and their uptime on NodePing account or subaccount.
    """
    if isinstance(checks, str) and checks.upper() == "ALL":
        url = "{}/{}".format(API_URL, ROUTE)
    else:
        url = "{}/{}?{}".format(
            API_URL, ROUTE, _utils.generate_querystring({"id": ",".join(checks)})
        )

    data = {"token": token, "uptime": True, "start": start}
    senddata = _utils.add_custid(data, customerid)

    return _utils.get(url, senddata)


def get_by_id(
    token: str, checkid: str, customerid: str | None = None
) -> checktypes.GetCheck:
    """Get a single NodePing check by ID.

    Args:
        token (str): NodePing API token
        checkid (str): Check ID
        customerid (str): subaccount ID

    Returns:
        dict: Contents of a single check by check ID
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, checkid)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def get_active(
    token: str, customerid: str | None = None
) -> dict[str, checktypes.GetCheck]:
    """Get active (enabled) checks on the NodePing account or subaccount.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: All enabled checks.
    """

    checks = get_all(token, customerid)

    return {k: v for k, v in checks.items() if v["enable"] == "active"}


def get_inactive(
    token: str, customerid: str | None = None
) -> dict[str, checktypes.GetCheck]:
    """Get inactive (disabled) checks on the NodePing account or subaccount.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: All disabled checks.
    """

    checks = get_all(token, customerid)

    return {k: v for k, v in checks.items() if v["enable"] == "inactive"}


def get_last_result(
    token: str, checkid: str, customerid: str | None = None
) -> checktypes.GetCheckUptime:
    """Get the last result for the specified check.

    Args:
        token (str): NodePing API token
        checkid (str): Check ID
        customerid (str): subaccount ID

    Returns:
        dict: check information with lastresult in the response
    """
    querystring = _utils.generate_querystring({"uptime": "true"})
    url = "{}/{}/{}?{}".format(API_URL, ROUTE, checkid, querystring)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def create_check(
    token: str, args, customerid: str | None = None
) -> checktypes.ModifiedCheck:
    """Create a NodePing check with parameters from specific dataclass check type.

    Args:
        token (str): NodePing API token
        args: a dataclass such as AgentCheck, HttpCheck, PingCheck, etc.
        customerid (str): subaccount ID

    Returns:
        dict: Contents of successfully created check or error message
    """
    url = "{}/{}".format(API_URL, ROUTE)
    data = asdict(args)
    data.update({"token": token, "customerid": customerid})

    return _utils.post(url, data)


def update_check(
    token: str,
    checkid: str,
    checktype: str,
    args: dict[str, str | int | bool | None],
    customerid: str | None = None,
) -> checktypes.ModifiedCheck:
    """Update an existing check on your NodePing account

    Args:
        token (str): NodePing API token
        checkid (str): The check ID that is being updated
        checktype (str): HTTP, PING, MTR, DNS, etc. check type
        args (dict): fields being updated with their new values
        customerid (str|None): subaccount ID

    Returns:
        dict: Contents of check ID with updated fields or error message
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, checkid)
    args.update({"type": checktype.upper(), "token": token, "customerid": customerid})

    return _utils.put(url, args)


def delete_check(
    token: str, checkid: str, customerid: str | None = None
) -> dict[str, str | bool]:
    """ """
    url = "{}/{}/{}".format(API_URL, ROUTE, checkid)
    senddata = _utils.add_custid({"token": token}, customerid)

    return _utils.delete(url, senddata)


def mute_check(
    token: str, checkid: str, duration: int | bool, customerid: str | None = None
) -> checktypes.ModifiedCheck:
    """Mute a NodePing check by check ID.

    Args:
        token (str): NodePing API token
        checkid (str): The check ID that is being updated
        duration (int|bool): True to mute infinitely, False to unmute, or a millisecond epoch timestamp in the future to specify when the check will be unmuted
        customerid (str|None): subaccount ID

    Returns:
        dict: check info
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, checkid)
    senddata = _utils.add_custid({"token": token, "mute": duration}, customerid)

    return _utils.put(url, senddata)


def disable_by(
    token: str,
    disabletype: str,
    string: str,
    disable: bool,
    customerid: str | None = None,
) -> dict[str, int]:
    """Find matching checks and disable it/them.

    This will not re-enable checks that were previously disabled
    using the 'enabled' element.

    Returned data:
        `disableall` - number of checks disabled after command
        `disabled` - number of checks currently disabled
        `enabled` - number of checks currently enabled

    Args:
        token (str): NodePing API token
        disabletype (str): "type", "label", or "target"
            * type: regex that matches against the 'type' field of checks
            * label: regex that matches against the 'label' field of checks
            * target: regex that matches against the 'target' field of checks
        string (str): the regex string to match the type, label, or target
        disable (bool): re-enable check (False) or disable the check (True)
        customerid (str|None): subaccount ID

    Returns:
        dict: check information {'disableall': x, 'disabled': y, 'enabled': z}
    """
    querystring = _utils.generate_querystring(
        {disabletype: string, "disableall": disable}
    )
    url = "{}/{}?{}".format(API_URL, ROUTE, querystring)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.put(url, data)


def disable_all(
    token: str, disable: bool, customerid: str | None = None
) -> dict[str, int]:
    """Disable all checks on the account.

    This will not re-enable checks that were previously disabled
    using the 'enabled' element

    Returned data:
        `disableall` - number of checks disabled after command
        `disabled` - number of checks currently disabled
        `enabled` - number of checks currently enabled

    Args:
        token (str): NodePing API token
        disable (bool): re-enable check (False) or disable the check (True)
        customerid (str|None): subaccount ID

    Returns:
        dict: check information {'disableall': x, 'disabled': y, 'enabled': z}
    """
    querystring = _utils.generate_querystring({"disableall": disable})
    url = "{}/{}?{}".format(API_URL, ROUTE, querystring)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.put(url, data)


def _parse_pass_fail(
    checks: dict[str, checktypes.GetCheck | checktypes.GetCheckUptime],
    pass_or_fail: int,
) -> dict:
    """Get all passing or failing active checks in NodePing result.

    Args:
        pass_or_fail (int): 1 for passing, 0 for failing

    Returns:
        dict: Dictionary of all passing or failing checks
    """
    result = {}

    for checkid, contents in checks.items():
        try:
            state = contents["state"]

            if contents["enable"] == "active" and state == pass_or_fail:
                result.update({checkid: contents})
        except KeyError:
            continue

    return result



# -*- coding: utf-8 -*-

from dataclasses import dataclass


@dataclass
class Results:
    """
    https://nodeping.com/docs-api-results.html#get

    Args:
        span (int): number of hours of results to retrieve. Cannot be used with start/end.
        limit (int): number of records to retrieve
        start (int | str): date/time for results to start. Millisecond, RFC2822, or ISO 8601. Times are assumed to be GMT unless the time includes the timezone
        end (int | str): use in conjunction with start. End time to get results
        clean (bool): whether to use the older format for results. True is recommended.
    """
    span: int | None = None
    limit: int = 300
    start: int | str | None = None
    end: int | str | None = None
    clean: bool = True


@dataclass
class Uptime:
    """
    https://nodeping.com/docs-api-results.html#uptime

    Args:
        interval (str): "days" or "months"
        start (str): start date of days or months (e.g. 2024-03-10). None starts at beginning of available data
        end (str): end date of days or months. None meands the end date is "now"
        offset (int): records are stored using UTC. Use this to offset the information for a different time zone. e.g. -24 to 24 hours offset.
    """
    offset: int | None = None
    interval: str = "months"
    start: str | None = None
    end: str | None = None


@dataclass
class Events:
    """
    https://nodeping.com/docs-api-results.html#events

    Records are sorted by the start time of event in descending order.

    Args:
        start (str): start date to retrieve events from a specific range of time
        end (str): end date to retrieve events from a specific range of time
        limit (int): limit for number of records to retrieve
    """
    start: int | str | None = None
    end: int | str | None = None
    limit: int | None = None



# -*- coding: utf-8 -*-

""" Manage scheduled or ad-hoc maintenance schedules.

Get, create, update, and delete maintenance schedules for your account.

https://nodeping.com/docs-api-maintenance.html
"""

from dataclasses import asdict
from .nptypes import maintenancetypes
from . import _utils
from ._utils import API_URL

ROUTE = "maintenance"


def get_all(token: str, customerid: str | None = None) -> dict:
    """Get information about all maintenances.

    https://nodeping.com/docs-api-maintenance.html#get

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: All maintenance information
    """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}".format(API_URL, ROUTE), data)


def get(token: str, maintenanceid: str, customerid: str | None = None) -> dict:
    """Get information about one maintenance.

    https://nodeping.com/docs-api-maintenance.html#get

    Args:
        token (str): NodePing API token
        maintenanceid (str): Maintenance ID
        customerid (str): subaccount ID

    Returns:
        dict: Maintenance ID's maintenance information
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, maintenanceid)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get(url, data)


def create(token: str, args, customerid: str | None = None) -> dict:
    """Create an new ad-hoc or scheduled maintenance.

    https://nodeping.com/docs-api-maintenance.html#post

    Args:
        token (str): NodePing API token
        args (dataclass): maintenancetypes.AdHocCreate or maintenancetypes.ScheduledCreate
        customerid (str): subaccount ID

    Returns:
        dict: Created maintenance's information
    """
    if isinstance(args, maintenancetypes.AdHocCreate):
        url = "{}/{}/ad-hoc".format(API_URL, ROUTE)
    elif isinstance(args, maintenancetypes.ScheduledCreate):
        url = "{}/{}".format(API_URL, ROUTE)
    else:
        return {"error": "Invalid maintenance class"}

    data = asdict(args)
    data["token"] = token
    senddata = _utils.add_custid(data, customerid)

    return _utils.post(url, senddata)


def update(token: str, id: str, args, customerid: str | None = None) -> dict:
    """Update an existing ad-hoc or scheduled maintenance.

    https://nodeping.com/docs-api-maintenance.html#put

    Args:
        token (str): NodePing API token
        id (str): Maintenance ID
        args (dataclass): maintenancetypes.AdHocUpdate or maintenancetypes.ScheduledUpdate
        customerid (str): subaccount ID

    Returns:
        dict: Updated maintenance's information
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, id)

    data = asdict(args)
    data["token"] = token
    senddata = _utils.add_custid(data, customerid)

    return _utils.put(url, senddata)


def delete(token: str, maintenanceid: str, customerid: str | None = None) -> dict:
    """Delete one maintenance.

    https://nodeping.com/docs-api-maintenance.html#delete

    Args:
        token (str): NodePing API token
        maintenanceid (str): Maintenance ID
        customerid (str): subaccount ID

    Returns:
        dict: Confirmation of maintenance successfully being deleted or not.
    """
    url = "{}/{}/{}".format(API_URL, ROUTE, maintenanceid)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.delete(url, data)



# -*- coding: utf-8 -*-

""" Get checks that were created on your NodePing account.

Allows you go get all checks, get passing, failing, by its ID,
disabled checks, and last results for a check.
"""

from . import _utils
from ._utils import API_URL


def get_all_probes(token: str) -> dict:
    """Get information on all NodePing probes


    The list of probes can be found at our FAQ:
    https://nodeping.com/faq.html

    Args:
        token (str): NodePing API token

    Returns:
        dict: Dictionary of NodePing probe information
    """
    url = "{}/info/probe".format(API_URL)

    return _utils.get(url, {"token": token})


def get_probe(token: str, location: str) -> dict:
    """Get information for a single NodePing probe

    The list of probes can be found at our FAQ:
    https://nodeping.com/faq.html

    Args:
        token (str): NodePing API token
        location (str): 2-letter abbreviation for the probe

    Returns:
        dict: Dictionary of single NodePing probe information
    """
    url = "{}/info/probe/{}".format(API_URL, location)

    return _utils.get(url, {"token": token})


def get_all_locations(token: str) -> dict:
    """Get information on all NodePing regions/locations


    The list of locations can be found at our FAQ:
    https://nodeping.com/faq.html

    Args:
        token (str): NodePing API token

    Returns:
        dict: Dictionary of NodePing location information
    """
    url = "{}/info/location".format(API_URL)

    return _utils.get(url, {"token": token})


def get_location(token: str, location: str) -> dict:
    """Get information for a single NodePing region/location

    The list of locations can be found at our FAQ:
    https://nodeping.com/faq.html

    Args:
        token (str): NodePing API token
        location (str): 3-letter abbreviation for the location

    Returns:
        dict: Dictionary of single NodePing location information
    """
    url = "{}/info/location/{}".format(API_URL, location)

    return _utils.get(url, {"token": token})



# -*- coding: utf-8 -*-

"""
Get, create, update, and delete schedules for notifications.
"""

from . import _utils
from ._utils import API_URL

ROUTE = "schedules"


def get_all(token, customerid: str | None = None) -> dict:
    """ """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}".format(API_URL, ROUTE), data)


def get(token: str, schedule: str, customerid: str | None = None) -> dict:
    """ """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}/{}".format(API_URL, ROUTE, schedule), data)


def create(
    token: str, name: str, schedule: dict, customerid: str | None = None
) -> dict:
    """Create a new notification schedule for the specified NodePing account.

    Args:
        token (str): NodePing API token
        name (str): The name of the schedule
        schedule (dict): The schedules for each day for receiving notifications
        customerid (str | None): subaccount ID

    Returns:
        dict: Entry for newly created schedule

    Example:

    {'data': {'friday': {'disabled': True},
              'monday': {'allday': True},
              'saturday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'sunday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'thursday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'tuesday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'wednesday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'}}}

    Days accept certain variables certain key/value pairs such as:
    time1: str - start of timespan (24-hour time)
    time2: str - end of timespan (24-hour time)
    exclude: True/False - inverts the time span so it is all day
    except for the time between time1 and time2
    disabled: True/False - disables notifications for this day.
    allday: True/False - enables notifications for the entire day.
    """
    schedule["token"] = token
    data = _utils.add_custid(schedule, customerid)

    return _utils.post("{}/{}/{}".format(API_URL, ROUTE, name), data)


def update(
    token: str, name: str, schedule: dict, customerid: str | None = None
) -> dict:
    """Update a new notification schedule for the specified NodePing account.

    Args:
        token (str): NodePing API token
        name (str): The name of the schedule
        schedule (dict): The schedules for each day for receiving notifications
        customerid (str | None): subaccount ID

    Returns:
        dict: Entry for updated schedule

    Example:

    {'data': {'friday': {'disabled': True},
              'monday': {'allday': True},
              'saturday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'sunday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'thursday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'tuesday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'},
              'wednesday': {'exclude': False, 'time1': '6:00', 'time2': '18:00'}}}

    Days accept certain variables certain key/value pairs such as:
    time1: str - start of timespan (24-hour time)
    time2: str - end of timespan (24-hour time)
    exclude: True/False - inverts the time span so it is all day
    except for the time between time1 and time2
    disabled: True/False - disables notifications for this day.
    allday: True/False - enables notifications for the entire day.
    """
    schedule["token"] = token
    data = _utils.add_custid(schedule, customerid)

    return _utils.put("{}/{}/{}".format(API_URL, ROUTE, name), data)


def delete(token: str, schedule: str, customerid: str | None = None) -> dict:
    """ """
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.delete("{}/{}/{}".format(API_URL, ROUTE, schedule), data)



# -*- coding: utf-8 -*-

"""NodePing contact types"""


from typing import Optional, TypedDict


class GenericAddress(TypedDict):
    accountsuppressall: bool
    address: str
    status: str
    mute: Optional[bool | int]
    suppressdown: Optional[bool]
    suppresup: Optional[bool]
    suppressdiag: Optional[bool]


class EmailAddress(GenericAddress):
    type: str


class SmsAddress(GenericAddress):
    type: str


class VoiceAddress(GenericAddress):
    type: str


class WebhookAddress(GenericAddress):
    action: str
    data: dict[str, str]
    headers: dict[str, str]
    type: str


class PagerDutyAddress(GenericAddress):
    type: str


class PushoverAddress(GenericAddress):
    priority: int
    type: str


class SlackAddress(GenericAddress):
    type: str


class HipChatAddress(GenericAddress):
    type: str


class ManyContacts(TypedDict):
    addresses: dict[
        str,
        EmailAddress
        | SmsAddress
        | VoiceAddress
        | WebhookAddress
        | PagerDutyAddress
        | PushoverAddress
        | SlackAddress
        | HipChatAddress,
    ]
    customer_id: str
    custrole: str
    name: str
    type: str


class Contact(ManyContacts):
    _id: str



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

NAME = "nodepingpy"
DESCRIPTION = "Python package for querying the NodePing API"
URL = "https://github.com/NodePing/nodepingpy"
EMAIL = "support@nodeping.com"
AUTHOR = "NodePing"
VERSION = "1.0.2"
LICENSE = "MIT"

setuptools.setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=URL,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-

"""
Manage your account and subaccounts via the API, as referenced
in the NodePing documentation.

https://nodeping.com/docs-api-accounts.html
"""

from dataclasses import asdict, dataclass

from . import _utils
from ._utils import API_URL

ROUTE = "accounts"


@dataclass
class Account:
    """Account creation args.

    Args:
        name (str): Name for the subaccount
        contactname (str): name for the primary contact of the subaccount
        email (str): email address of the primary contact for the subaccount
        timezone (str): GMT offset for the subaccount ("-7", "+3", etc.)
        location (str): Default region for checks ('eur', 'nam', 'lam', 'eao', 'wlw')
        emailme (bool): True to opt-in the subaccount for service email notifications
        autodiagnotifications (bool): enable/disable account-wide emails for automated diagnostics
    """

    name: str
    contactname: str
    email: str
    timezone: str
    location: str
    emailme: bool
    autodiagnotifications: bool


@dataclass
class AccountUpdate:
    """Account update args.

    Args:
        name (str): Name for the subaccount
        timezone (str): GMT offset for the subaccount ("-7", "+3", etc.)
        location (str): Default region for checks ('eur', 'nam', 'lam', 'eao', 'wlw')
        emailme (bool): True to opt-in the subaccount for service email notifications
        status (str): "Active" or "Suspend", not supported for parent accounts
        autodiagnotifications (bool): enable/disable account-wide emails for automated diagnostics
    """

    name: str | None = None
    timezone: str | None = None
    location: str | None = None
    emailme: bool | None = None
    status: str | None = None
    autodiagnotifications: bool | None = None


def info(token: str, customerid: str | None = None) -> dict[str, str | int | bool]:
    """Returns the account information.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: Account information
    """
    data = _utils.add_custid({"token": token}, customerid)
    return _utils.get("{}/{}".format(API_URL, ROUTE), data)


def is_valid(token: str, customerid: str | None = None) -> bool:
    """Returns if you API key is valid or not.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        bool: True if valid token, False if not
    """
    account_info = info(token, customerid)

    try:
        account_info["error"]
    except KeyError:
        return True
    else:
        return False


def create_subaccount(token: str, args: Account) -> dict[str, str | int | bool]:
    """Create a subaccount under your NodePing account.

    Args:
        token (str): NodePing API token
        args: (Account): Account class containing account information

    Returns:
        dict: Data returned from NodePing API
    """
    data = asdict(args)
    data["token"] = token

    return _utils.post("{}/{}".format(API_URL, ROUTE), data)


def update_account(
    token: str, args: AccountUpdate, customerid: str | None = None
) -> dict[str, str | int | bool]:
    """Update a NodePing account or subaccount.

    Args:
        token (str): NodePing API token
        args: (AccountUpdate): AccountUpdate class containing account information

    Returns:
        dict: Data returned from NodePing API
    """
    data = asdict(args)
    data["token"] = token
    put_data = _utils.add_custid(data, customerid)

    return _utils.put("{}/{}".format(API_URL, ROUTE), put_data)


def delete_subaccount(token: str, customerid: str):
    """Delete a NodePing subaccount.

    Args:
        token (str): NodePing API token
        customerid (str): subaccount ID

    Returns:
        dict: Confirmation of subaccount deletion success/failure
    """
    url = "{}/{}".format(API_URL, ROUTE)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.delete(url, data)


def disable_notifications(
    token: str, accountsupressall: bool, customerid: str | None = None
) -> dict[str, str | int | bool]:
    """Disable notifications on an account or subaccount.

    Args:
        token (str): NodePing API token
        accountsupressall (bool): True to disable notifcations, False to enable
        customerid (str): subaccount ID

    Returns:
        dict: Response from NodePing API
    """
    querystring = _utils.generate_querystring(
        {"accountsupressall": str(accountsupressall).lower()}
    )
    url = "{}/{}?{}".format(API_URL, ROUTE, querystring)
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.put(url, data)



# -*- coding: utf-8 -*-

from dataclasses import asdict
from . import _utils
from .nptypes import resulttypes
from ._utils import API_URL


def get(token: str, id: str, args, customerid: str | None = None) -> dict:
    """ 
    https://nodeping.com/docs-api-results.html#get

    Args:
        token (str): NodePing API token
        id (str): ID of the check for which you are retrieving events
        args (dict): nptypes.resulttypes dataclass for Results, Uptime, or Events
        customerid (str | None): subaccount ID

    Returns:
        dict: API return data
    """
    data = asdict(args)
    data["token"] = token
    senddata = _utils.add_custid(data, customerid)

    if isinstance(args, resulttypes.Results):
        route = "results"
    elif isinstance(args, resulttypes.Uptime):
        route = "results/uptime"
    elif isinstance(args, resulttypes.Events):
        route = "results/events"
    else:
        return {"error": "args not a valid data type"}

    return _utils.get("{}/{}/{}".format(API_URL, route, id), senddata)


def get_current(token: str, customerid: str | None = None) -> dict:
    """ 
    https://nodeping.com/docs-api-results.html#uptime

    Args:
        token (str): NodePing API token
        customerid (str | None): subaccount ID

    Returns:
        dict: Current events for checks
    """
    route = "results/current"
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}".format(API_URL, route), data)


def get_summary(token: str, id: str, customerid: str | None = None) -> dict:
    """ 
    https://nodeping.com/docs-api-results.html#events

    Args:
        token (str): NodePing API token
        customerid (str | None): subaccount ID

    Returns:
        dict: Hourly summary information about the results for a check
    """
    route = "results/summary"
    data = _utils.add_custid({"token": token}, customerid)

    return _utils.get("{}/{}/{}".format(API_URL, route, id), data)
